@@ -164,6 +164,10 @@ void player::stop_playback()
         _video_output->exit_fullscreen();
         _video_output->prepare_next_frame(video_frame(), subtitle_box());
         _video_output->activate_next_frame();
+        if (_audio_output)
+        {
+            _audio_output->stop();
+        }
     }
     notify(notification::play, true, false);
 }
@@ -497,7 +501,7 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
             notify(notification::pause, false, true);
         }
         *more_steps = true;
-        return 0;
+        return 1000;    // allow some sleep in pause mode
     }
     else if (_need_frame_now)
     {
@@ -523,12 +527,11 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
         _video_pos = _video_frame.presentation_time;
         if (_media_input->selected_subtitle_stream() >= 0)
         {
-            // TODO: start subtitle reading asynchronously to use benefits of multithreading
             while (_next_subtitle_box.is_valid()
                     && _next_subtitle_box.presentation_stop_time < _video_pos)
             {
-                _media_input->start_subtitle_box_read();
                 _next_subtitle_box = _media_input->finish_subtitle_box_read();
+                _media_input->start_subtitle_box_read();
                 // If the box is invalid, we reached the end of the subtitle stream.
                 // Ignore this and let audio/video continue.
             }
@@ -682,11 +685,8 @@ bool player::run_step()
         _video_output->activate_next_frame();
     }
 
-    if (_video_output->has_events())
-    {
-        _video_output->process_events();
-    }
-    else if (allowed_sleep > 0)
+    _video_output->process_events();
+    if (allowed_sleep > 0)
     {
         usleep(allowed_sleep);
     }
